# **********************************************************************
# Copyright 2014 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************
import nsx_logging
import sys
from cli.utils.error_code import ErrorCode as ec


lg = nsx_logging.getLogger("cli.utils.error_utils")


ERROR_HEADER = "%"
_MESSAGES = {}  # Initialized by descriptor_factory.py


def set_messages(msgs):
    """Set messages in global message store."""
    global _MESSAGES
    _MESSAGES = msgs


def render_error(msg_id, *args):
    """Print error message given message id and optional message arguments."""
    # The following line is what the user sees when an error msg is rendered
    sys.stdout.write("%s %s" % (ERROR_HEADER, get_message(msg_id, *args)))
    sys.stdout.write("\n")  # empty line before showing prompt


def render_error_result(result, output_prefix=None):
    """Print error message given provider error output.

    Output prefix if not None, is then printed on stderr;
    before getting the message from msg_id's.

    Args:
        result: Provider error result in the form of:
            MSG_ID
            [optional MSG_ID arg 1]
            [optional MSG_ID arg 2] ... etc

            [optional provider output data]
    """
    if output_prefix is not None:
        sys.stderr.write(output_prefix)
        sys.stderr.write("\n")
        sys.stderr.flush()
    parts = result.strip().split("\n")
    msg_id = parts[0].strip()
    render_error(msg_id, *parts[1:])


def format_error(msg_id, *args):
    """Format error message given message id and optional message arguments."""
    parts = [msg_id]
    if args:
        parts += [str(arg) for arg in args]
    return "\n".join(parts)


def render_message(msg_id, *args):
    """Print message given message id and optional message arguments."""
    sys.stdout.write("%s\n" %  get_message(msg_id, *args))


def get_message(msg_id, *args):
    """Return message id and its optional arguments in string format."""
    if msg_id not in _MESSAGES:
        lg.error("Specified msg_id not in _MESSAGES: %s" % msg_id,
                 error_code=ec.OPERATION_FAILED)
        return ""
    message = _MESSAGES[msg_id]
    num_format_specifiers = message.count("%s")
    if len(args) != num_format_specifiers:
        msg = ("Provider returned incorrect format args (%s) for msg_id %s "
               "message: %s" % (args, msg_id, message))
        lg.error(msg, error_code=ec.OPERATION_FAILED)
        assert False, msg
        return ""
    return message % tuple(args)
