#!/bin/bash

if (( $# < 3 )); then
    echo "Illegal number of parameters" >&2
    exit 1
fi

action=$1
shift

while getopts :s:p:t: opt; do
    case $opt in
        s)
          server=$OPTARG
          ;;
        p)
          port=$OPTARG
          ;;
        t)
          protocol=$OPTARG
          ;;
        *)
          echo "invalid option $OPTARG" >&2;
          exit 1;
          ;;
    esac
done

if [ -z "$port" ] || [ -z "$protocol" ]; then
    echo "Port/protocol not specified" >&2
    exit 1
fi

interface='eth0'
user="admin"
rule=""
if [ ! -z "$interface" ]; then
    rule="$rule -o $interface"
fi
if [ ! -z "$server" ]; then
    rule="$rule -d $server"
fi
if [ "$protocol" == "TCP" ]; then
    rule="$rule -p tcp -m tcp --dport $port --tcp-flags FIN,SYN,RST,ACK SYN"
fi
if [ "$protocol" == "UDP" ]; then
    rule="$rule -p udp -m udp --dport $port"
fi
if [ ! -z "$user" ]; then
    rule="$rule -m owner --uid-owner $user"
fi
if [ ! -z "$group" ]; then
    rule="$rule -m owner --gid-owner $group"
fi
rule="$rule -j ACCEPT"
if [ $action == "set" ]; then
    cmd="iptables -I OUTPUT 4 $rule"
    ${cmd}
    ret=$?
    if [ "$ret" != 0 ]; then
        echo "Error creating iptable rule. cmd = $cmd" >&2
        exit $ret
    fi
    echo "Created iptable rule. cmd = $cmd"
elif [ "$action" == "delete" ]; then
    cmd="iptables -D OUTPUT $rule"
    ${cmd}
    ret=$?
    if [ "$ret" != 0 ]; then
        echo "Error deleting iptable rule. cmd = $cmd" >&2
        exit $ret
    fi
    echo "Deleted iptable rule. cmd = $cmd"
fi

exit $ret
