# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential

import platform

import sha.config
from sha.contrib.monitor.abc_base import Monitor
from sha.config import ENABLED_MONITORS
from sha.constants import MONITER_METRIC_REPORTED, MONITER_METRIC_FAIL_REPORT

class UplinkStatusMonitor(Monitor):
    """
    UplinkStatusMonitor is a kind of Monitor which monitors the uplink status
    change on dataplane.
    """

    def __init__(self, *args, **kwargs):
        self._logger = sha.config.get_logger(__name__)
        self._internal_monitor = None
        self._init_internal_monitor()
        self._metric_name = ENABLED_MONITORS['UplinkStatusMonitor']
        self._stats = {}

    def _init_internal_monitor(self):
        if platform.system() == sha.config.PLATFORM_ESX:
            from sha.contrib.monitor.uplinkstatus.esx import \
                EsxUplinkStatusMonitor
            self._internal_monitor = EsxUplinkStatusMonitor()
            self._logger.info("UplinkStatusMonitor initialized, internal monitor: {}".format("EsxUplinkStatusMonitor"))
        else:
            from sha.contrib.monitor.uplinkstatus.ovs import \
                OvsUplinkStatusMonitor
            self._internal_monitor = OvsUplinkStatusMonitor()
            self._logger.info("UplinkStatusMonitor initialized, internal monitor: {}".format("OvsUplinkStatusMonitor"))

    def _stats_inc(self, field):
        self._stats[field] =  self._stats.get(field, 0) + 1

    def _notify_whiteboard(self, *args):
        from sha.core.whiteboard import DataWhiteboard
        if not DataWhiteboard.report_data(self._metric_name, args):
             self._logger.warning("Failed to report metric {} to whiteboard".format(metric_key))
             self._stats_inc(MONITER_METRIC_REPORTED)
        else:
             self._stats_inc(MONITER_METRIC_FAIL_REPORT)

    def start(self):
        # if notify not set, notify to whiteboard
        if not self._internal_monitor.notify:
            self._internal_monitor.notify = self._notify_whiteboard
        self._internal_monitor.start()

    def stop(self):
        self._internal_monitor.stop()

    def join(self):
        self._logger.info("UplinkStatusMonitor join not implemented yet.")

    @property
    def internal_monitor(self):
        return self._internal_monitor

    @property
    def metric_name(self):
        return self._metric_name



